package j1.tutorial.javac.analysis.infrastructure;

import com.sun.source.util.TaskEvent;
import com.sun.source.util.TaskEvent.Kind;
import com.sun.source.util.TaskListener;
import com.sun.source.util.Trees;
import com.sun.tools.javac.processing.JavacProcessingEnvironment;
import com.sun.tools.javac.util.Context;
import com.sun.tools.javac.util.Options;
import j1.tutorial.javac.analysis.api.AnalysisContext;
import j1.tutorial.javac.analysis.api.Rule;
import java.util.ServiceLoader;
import java.util.Set;
import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.Processor;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.TypeElement;
import org.openide.util.lookup.ServiceProvider;

/**Only for NetBeans (7.3)
 * 
 * @author lahvac
 */
@SupportedAnnotationTypes("*")
@ServiceProvider(service=Processor.class)
public class NBAPInvoker extends AbstractProcessor {

    @Override
    public boolean process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv) {
        if (roundEnv.processingOver()) {
            if (SourceVersion.latest().compareTo(SourceVersion.RELEASE_7) <= 0) {
                //NetBeans 7.2:
                Context context = ((JavacProcessingEnvironment) processingEnv).getContext();
                if (Options.instance(context).isSet("ide")) {
                    TaskListener origTL = context.get(TaskListener.class);
                    if (origTL != null) context.put(TaskListener.class, (TaskListener) null);
                    context.put(TaskListener.class, new TaskListenerImpl(Trees.instance(processingEnv), origTL));
                }
            }
        }
        
        return false;
    }
    
    private final class TaskListenerImpl implements TaskListener {

        private final Trees trees;
        private final TaskListener origTL;

        public TaskListenerImpl(Trees trees, TaskListener origTL) {
            this.trees = trees;
            this.origTL = origTL;
        }
        
        @Override public void started(TaskEvent te) {
            if (origTL != null) origTL.started(te);
        }

        @Override public void finished(TaskEvent te) {
            if (origTL != null) origTL.finished(te);
            if (te.getKind() == Kind.ANALYZE) {
                for (Rule r : ServiceLoader.load(Rule.class, NBAPInvoker.class.getClassLoader())) {
                    r.analyseFile(new AnalysisContext(processingEnv, trees), te.getCompilationUnit());
                }
            }
        }
        
    }
    
}
