package j1.tutorial.javac.analysis.api;

import com.sun.source.tree.AnnotationTree;
import com.sun.source.tree.CatchTree;
import com.sun.source.tree.ClassTree;
import com.sun.source.tree.CompilationUnitTree;
import com.sun.source.tree.ExpressionTree;
import com.sun.source.tree.ImportTree;
import com.sun.source.tree.LineMap;
import com.sun.source.tree.MethodInvocationTree;
import com.sun.source.tree.MethodTree;
import com.sun.source.tree.ParameterizedTypeTree;
import com.sun.source.tree.Scope;
import com.sun.source.tree.Tree;
import com.sun.source.tree.TreeVisitor;
import com.sun.source.util.JavacTask;
import com.sun.source.util.SourcePositions;
import com.sun.source.util.TreePath;
import com.sun.source.util.Trees;
import com.sun.tools.javac.tree.JCTree;
import com.sun.tools.javac.tree.JCTree.JCClassDecl;
import com.sun.tools.javac.tree.JCTree.JCCompilationUnit;
import com.sun.tools.javac.tree.JCTree.JCFieldAccess;
import com.sun.tools.javac.tree.JCTree.JCIdent;
import com.sun.tools.javac.tree.JCTree.JCMethodDecl;
import com.sun.tools.javac.tree.JCTree.JCNewClass;
import com.sun.tools.javac.tree.JCTree.JCVariableDecl;
import java.util.List;
import javax.annotation.processing.Messager;
import javax.annotation.processing.ProcessingEnvironment;
import javax.lang.model.element.AnnotationMirror;
import javax.lang.model.element.AnnotationValue;
import javax.lang.model.element.Element;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.type.DeclaredType;
import javax.lang.model.type.ErrorType;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.util.Elements;
import javax.lang.model.util.Types;
import javax.tools.Diagnostic.Kind;
import javax.tools.JavaCompiler.CompilationTask;
import javax.tools.JavaFileObject;

public class AnalysisContext {

    private final Trees trees;
    private final Elements elements;
    private final Types types;
    private final Messager messager;
    
    public AnalysisContext(CompilationTask task) {
        this.trees = new EnhancedTrees(Trees.instance(task));
        this.elements = ((JavacTask) task).getElements();
        this.types = ((JavacTask) task).getTypes();
        this.messager = new MessagerImpl(trees);
    }
    
    public AnalysisContext(ProcessingEnvironment processingEnvironment, Trees trees) {
        this.trees = new EnhancedTrees(trees);
        this.elements = processingEnvironment.getElementUtils();
        this.types = processingEnvironment.getTypeUtils();
        this.messager = processingEnvironment.getMessager();
    }

    public Trees getTrees() {
        return trees;
    }

    public Elements getElementUtils() {
        return elements;
    }
    
    public Types getTypeUtils() {
        return types;
    }

    public Messager getMessager() {
        return messager;
    }

    private static class MessagerImpl implements Messager {
        
        private final Trees trees;

        public MessagerImpl(Trees trees) {
            this.trees = trees;
        }

        @Override
        public void printMessage(Kind kind, CharSequence msg) {
            printMessage(kind, msg, null);
        }

        @Override
        public void printMessage(Kind kind, CharSequence msg, Element e) {
            printMessage(kind, msg, e, null);
        }

        @Override
        public void printMessage(Kind kind, CharSequence msg, Element e, AnnotationMirror a) {
            printMessage(kind, msg, e, a, null);
        }

        @Override
        public void printMessage(Kind kind, CharSequence msg, Element e, AnnotationMirror a, AnnotationValue v) {
            TreePath path = trees.getPath(e, a, v);
            
            if (path != null) {
                trees.printMessage(kind, msg, path.getLeaf(), path.getCompilationUnit());
            } else {
                trees.printMessage(kind, msg, null, new FakeCompilationUnit());
            }
        }

        private static class FakeCompilationUnit implements CompilationUnitTree {
            @Override public List<? extends AnnotationTree> getPackageAnnotations() {
                throw new UnsupportedOperationException("Not supported yet.");
            }
            @Override public ExpressionTree getPackageName() {
                throw new UnsupportedOperationException("Not supported yet.");
            }
            @Override public List<? extends ImportTree> getImports() {
                throw new UnsupportedOperationException("Not supported yet.");
            }
            @Override public List<? extends Tree> getTypeDecls() {
                throw new UnsupportedOperationException("Not supported yet.");
            }
            @Override public JavaFileObject getSourceFile() {
                return null;
            }
            @Override public LineMap getLineMap() {
                throw new UnsupportedOperationException("Not supported yet.");
            }
            @Override public Tree.Kind getKind() {
                throw new UnsupportedOperationException("Not supported yet.");
            }
            @Override public <R, D> R accept(TreeVisitor<R, D> tv, D d) {
                throw new UnsupportedOperationException("Not supported yet.");
            }
        }
    }
    
    private static final class EnhancedTrees extends Trees {
        private final Trees delegate;

        public EnhancedTrees(Trees delegate) {
            this.delegate = delegate;
        }

        @Override
        public SourcePositions getSourcePositions() {
            return delegate.getSourcePositions();
        }

        @Override
        public Tree getTree(Element elmnt) {
            return delegate.getTree(elmnt);
        }

        @Override
        public ClassTree getTree(TypeElement te) {
            return delegate.getTree(te);
        }

        @Override
        public MethodTree getTree(ExecutableElement ee) {
            return delegate.getTree(ee);
        }

        @Override
        public Tree getTree(Element elmnt, AnnotationMirror am) {
            return delegate.getTree(elmnt, am);
        }

        @Override
        public Tree getTree(Element elmnt, AnnotationMirror am, AnnotationValue av) {
            return delegate.getTree(elmnt, am, av);
        }

        @Override
        public TreePath getPath(CompilationUnitTree cut, Tree tree) {
            return delegate.getPath(cut, tree);
        }

        @Override
        public TreePath getPath(Element elmnt) {
            return delegate.getPath(elmnt);
        }

        @Override
        public TreePath getPath(Element elmnt, AnnotationMirror am) {
            return delegate.getPath(elmnt, am);
        }

        @Override
        public TreePath getPath(Element elmnt, AnnotationMirror am, AnnotationValue av) {
            return delegate.getPath(elmnt, am, av);
        }

        private Element elementFor(Tree tree) {
            switch (tree.getKind()) {
                case COMPILATION_UNIT:
                    return ((JCCompilationUnit) tree).packge;
                case ANNOTATION_TYPE: case CLASS: case ENUM: case INTERFACE:
                    return ((JCClassDecl) tree).sym;
                case METHOD:
                    return ((JCMethodDecl) tree).sym;
                case VARIABLE:
                    return ((JCVariableDecl) tree).sym;
                case IDENTIFIER:
                    return ((JCIdent) tree).sym;
                case MEMBER_SELECT:
                    return ((JCFieldAccess) tree).sym;
                case NEW_CLASS:
                    return ((JCNewClass) tree).constructor;
                case METHOD_INVOCATION:
                    return elementFor(((MethodInvocationTree) tree).getMethodSelect());
                case PARAMETERIZED_TYPE:
                    return elementFor(((ParameterizedTypeTree) tree).getType());
                case ANNOTATION:
                case TYPE_PARAMETER:
                    if (((JCTree) tree).type != null)
                        return ((JCTree) tree).type.tsym;
            }
            
            return null;
        }
        
        @Override
        public Element getElement(TreePath tp) {
            return elementFor(tp.getLeaf());
        }

        @Override
        public TypeMirror getTypeMirror(TreePath tp) {
            return delegate.getTypeMirror(tp);
        }

        @Override
        public Scope getScope(TreePath tp) {
            return delegate.getScope(tp);
        }

        @Override
        public String getDocComment(TreePath tp) {
            return delegate.getDocComment(tp);
        }

        @Override
        public boolean isAccessible(Scope scope, TypeElement te) {
            return delegate.isAccessible(scope, te);
        }

        @Override
        public boolean isAccessible(Scope scope, Element elmnt, DeclaredType dt) {
            return delegate.isAccessible(scope, elmnt, dt);
        }

        @Override
        public TypeMirror getOriginalType(ErrorType et) {
            return delegate.getOriginalType(et);
        }

        @Override
        public void printMessage(Kind kind, CharSequence cs, Tree tree, CompilationUnitTree cut) {
            delegate.printMessage(kind, cs, tree, cut);
        }

        @Override
        public TypeMirror getLub(CatchTree ct) {
            return delegate.getLub(ct);
        }
        
    }
    
}
