/*
package j1.tutorial.javac.analysis.rules;

import j1.tutorial.javac.analysis.api.RuleTest;
import org.testng.annotations.Test;

public class CollectionRemoveNGTest {
    
    public CollectionRemoveNGTest() {
    }

    @Test
    public void testSimple1() throws Exception {
        RuleTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<String> l = null;" +
                       "        l.remove(new Object());" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings("/test/Test.java:115-127:Suspicious parameter type: java.lang.Object, should be assignable to: java.lang.String");
    }

    @Test
    public void testSimple3() throws Exception {
        RuleTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<String> l = null;" +
                       "        l.remove(Integer.valueOf(1));" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings("/test/Test.java:115-133:Suspicious parameter type: java.lang.Integer, should be assignable to: java.lang.String");
    }

    @Test
    public void testPass() throws Exception {
        RuleTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<String> l = null;" +
                       "        l.remove(\"\");" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings();
    }
    
    @Test
    public void testSuperWildcard() throws Exception {
        RuleTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<? super String> l = null;" +
                       "        l.remove(\"\");" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings();
    }

    @Test
    public void testWildcard() throws Exception {
        RuleTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<?> l = null;" +
                       "        l.remove(\"\");" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings();
    }

    @Test
    public void testCollectionRemoveInteger() throws Exception {
        RuleTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<String> l = null;" +
                       "        l.remove(0);" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings();
    }

    @Test
    public void testCollectionRemoveInteger2() throws Exception {
        RuleTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<String> l = null;" +
                       "        l.subList(0, 0).remove(0);" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings();
    }

//    @Test
//    public void testExtends1() throws Exception {
//        RuleTest.create()
//                .input("package test;" +
//                       "public class Test extends java.util.LinkedList<String> {" +
//                       "    private void test () {" +
//                       "        remove(new Object());" +
//                       "    }" +
//                       "}")
//                .run(CollectionRemove.class)
//                .assertWarnings("/test/Test.java:115-127:Suspicious parameter type: java.lang.Object, should be assignable to: java.lang.String");
//    }
//
//    @Test
//    public void testExtends2() throws Exception {
//        RuleTest.create()
//                .input("package test;" +
//                       "public class Test extends java.util.LinkedList<String> {" +
//                       "    private void test () {" +
//                       "        new Runnable() {" +
//                       "            public void run() {" +
//                       "                remove(new Object());" +
//                       "            }" +
//                       "        };" +
//                       "    }" +
//                       "}")
//                .run(CollectionRemove.class)
//                .assertWarnings("0:166-0:186:verifier:SC: java.util.Collection.remove, Object, String");
//    }
//
//    @Test
//    public void testInsideItsOverride196606() throws Exception {
//        RuleTest.create()
//                .input("package test;" +
//                       "public class Test extends java.util.ArrayList<String> {" +
//                       "    @Override public boolean remove(Object o) {" +
//                       "        return super.remove(o);\n" +
//                       "    }" +
//                       "}")
//                .run(CollectionRemove.class)
//                .assertWarnings();
//    }
}
*/
