package j1.tutorial.javac.analysis.netbeans;

import com.sun.source.tree.ExpressionTree;
import com.sun.source.tree.MethodInvocationTree;
import com.sun.source.util.TreePath;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.type.DeclaredType;
import javax.lang.model.type.ExecutableType;
import javax.lang.model.type.TypeKind;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.type.WildcardType;
import org.netbeans.spi.editor.hints.ErrorDescription;
import org.netbeans.spi.java.hints.ConstraintVariableType;
import org.netbeans.spi.java.hints.ErrorDescriptionFactory;
import org.netbeans.spi.java.hints.Hint;
import org.netbeans.spi.java.hints.HintContext;
import org.netbeans.spi.java.hints.TriggerPattern;
import org.openide.util.NbBundle.Messages;

@Hint(displayName = "#DN_CollectionRemove", description = "#DESC_CollectionRemove", category = "general")
@Messages({
    "DN_CollectionRemove=Collection Remove",
    "DESC_CollectionRemove=Collection Remove"
})
public class CollectionRemove {

    @TriggerPattern(value = "$collection.remove($val)",
                    constraints = {
                        @ConstraintVariableType(variable = "$collection", type = "java.util.Collection"),
                        @ConstraintVariableType(variable = "$val", type = "java.lang.Object")
                    })
    public static ErrorDescription computeWarning(final HintContext ctx) {
        TreePath collection = ctx.getVariables().get("$collection");
        if (collection == null) return null;
        
        final ExecutableElement checkAgainst = Utilities.resolveMethod(ctx, "java.util.Collection.add(java.lang.Object)");
        TypeMirror site = ctx.getInfo().getTrees().getTypeMirror(collection);

        if (   site != null
            && site.getKind() == TypeKind.DECLARED) {
            TypeMirror actualMethodType = ctx.getInfo().getTypes().asMemberOf((DeclaredType) site, checkAgainst);
            TypeMirror expectedType = ((ExecutableType) actualMethodType).getParameterTypes().get(0);

            if (expectedType.getKind() == TypeKind.WILDCARD) {
                expectedType = ((WildcardType) expectedType).getExtendsBound();
                if (expectedType == null) {
                    expectedType = ctx.getInfo().getElements().getTypeElement("java.lang.Object").asType();
                }
            }
            
            ExpressionTree argument = ((MethodInvocationTree) ctx.getPath().getLeaf()).getArguments().get(0);
            TypeMirror actualType = ctx.getInfo().getTrees().getTypeMirror(new TreePath(ctx.getPath(), argument));

            if (!ctx.getInfo().getTypes().isAssignable(actualType, expectedType)) {
                return ErrorDescriptionFactory.forTree(ctx, argument, "Suspicious parameter type: " + actualType + ", should be assignable to: " + expectedType);
            }
        }

        return null;
    }

}
