package j1.tutorial.javac.analysis.netbeans;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.element.VariableElement;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.util.ElementFilter;
import org.netbeans.spi.java.hints.HintContext;

public class Utilities {
    private static final Pattern SPLIT = Pattern.compile("(.+)\\.([^.]+)\\((.*)\\)");
    
    public static ExecutableElement resolveMethod(HintContext ctx, String name) {
        Matcher m = SPLIT.matcher(name);

        if (!m.matches()) {
            throw new IllegalArgumentException();
        }

        String className = m.group(1);
        String methodName = m.group(2);
        String paramsSpec = m.group(3);

        TypeElement te = ctx.getInfo().getElements().getTypeElement(className);

        if (te == null) {
            return null;
        }

        String[] paramList = paramsSpec.split(",");
        List<TypeMirror> params = new LinkedList<TypeMirror>();

        for (String t : paramList) {
            TypeElement param = ctx.getInfo().getElements().getTypeElement(t);
            params.add(ctx.getInfo().getTypes().erasure(param.asType()));
        }

        for (ExecutableElement ee : ElementFilter.methodsIn(te.getEnclosedElements())) {
            if (!methodName.equals(ee.getSimpleName().toString()) || ee.getParameters().size() != params.size()) {
                continue;
            }
            
            Iterator<TypeMirror> designed = params.iterator();
            boolean found = true;

            for (VariableElement param : ee.getParameters()) {
                if (!ctx.getInfo().getTypes().isSameType(ctx.getInfo().getTypes().erasure(param.asType()), designed.next())) {
                    found = false;
                    break;
                }
            }

            if (found) {
                return ee;
            }
        }
        
        return null;
    }
    
}
