package j1.tutorial.javac.analysis.netbeans;

import org.junit.Test;
import org.netbeans.junit.NbTestCase;
import org.netbeans.modules.java.hints.test.api.HintTest;

public class CollectionRemoveTest extends NbTestCase {
    
    public CollectionRemoveTest(String name) {
        super(name);
    }

    @Test
    public void testSimple1() throws Exception {
        HintTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<String> l = null;" +
                       "        l.remove(new Object());" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings("0:115-0:127:verifier:Suspicious parameter type: java.lang.Object, should be assignable to: java.lang.String");
    }

    @Test
    public void testSimple3() throws Exception {
        HintTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<String> l = null;" +
                       "        l.remove(Integer.valueOf(1));" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings("0:115-0:133:verifier:Suspicious parameter type: java.lang.Integer, should be assignable to: java.lang.String");
    }

    @Test
    public void testPass() throws Exception {
        HintTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<String> l = null;" +
                       "        l.remove(\"\");" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings();
    }
    
    @Test
    public void testSuperWildcard() throws Exception {
        HintTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<? super String> l = null;" +
                       "        l.remove(\"\");" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings();
    }

    @Test
    public void testWildcard() throws Exception {
        HintTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<?> l = null;" +
                       "        l.remove(\"\");" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings();
    }

    @Test
    public void testCollectionRemoveInteger() throws Exception {
        HintTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<String> l = null;" +
                       "        l.remove(0);" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings();
    }

    @Test
    public void testCollectionRemoveInteger2() throws Exception {
        HintTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        java.util.List<String> l = null;" +
                       "        l.subList(0, 0).remove(0);" +
                       "    }" +
                       "}")
                .run(CollectionRemove.class)
                .assertWarnings();
    }
    
    @Test
    public void testCollectionRemoveError() throws Exception {
        HintTest.create()
                .input("package test;" +
                       "public class Test {" +
                       "    private void test () {" +
                       "        Undefined l = null;" +
                       "        l.remove(\"\");" +
                       "    }" +
                       "}", 
                       false)
                .run(CollectionRemove.class)
                .assertWarnings();
    }

//    @Test
//    public void testExtends1() throws Exception {
//        HintTest.create()
//                .input("package test;" +
//                       "public class Test extends java.util.LinkedList<String> {" +
//                       "    private void test () {" +
//                       "        remove(new Object());" +
//                       "    }" +
//                       "}")
//                .run(CollectionRemove.class)
//                .assertWarnings("0:115-0:127:verifier:Suspicious parameter type: java.lang.Object, should be assignable to: java.lang.String");
//    }
//
//    @Test
//    public void testExtends2() throws Exception {
//        HintTest.create()
//                .input("package test;" +
//                       "public class Test extends java.util.LinkedList<String> {" +
//                       "    private void test () {" +
//                       "        new Runnable() {" +
//                       "            public void run() {" +
//                       "                remove(new Object());" +
//                       "            }" +
//                       "        };" +
//                       "    }" +
//                       "}")
//                .run(CollectionRemove.class)
//                .assertWarnings("0:166-0:186:verifier:Suspicious parameter type: java.lang.Object, should be assignable to: java.lang.String");
//    }
//
//    @Test
//    public void testInsideItsOverride196606() throws Exception {
//        HintTest.create()
//                .input("package test;" +
//                       "public class Test extends java.util.ArrayList<String> {" +
//                       "    @Override public boolean remove(Object o) {" +
//                       "        return super.remove(o);\n" +
//                       "    }" +
//                       "}")
//                .run(CollectionRemove.class)
//                .assertWarnings();
//    }
}
